package org.bitbucket.bradleysmithllc.etlunit.feature.database;

import org.bitbucket.bradleysmithllc.etlunit.TestExecutionError;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;

import java.io.File;
import java.sql.Connection;
import java.util.List;

public interface DatabaseImplementation {
	enum operation {
		createDatabase,
		completeDatabaseInitialization,
		prepareDatabase
	}

	interface BaseRequest
	{
		String getMode();
		DatabaseConnection getConnection();
		ETLTestMethod getTestMethod();
	}

	interface InitializeRequest extends BaseRequest
	{
		File resolveSchemaReference(String ddlRef);
	}

	interface PrepareRequest extends BaseRequest
	{
		ETLTestMethod getTestMethod();
	}

	interface OperationRequest
	{
		InitializeRequest getInitializeRequest() throws UnsupportedOperationException;
		PrepareRequest getPrepareRequest() throws UnsupportedOperationException;
	}

	enum database_state
	{
		pass,
		fail
	}

	database_state getDatabaseState(DatabaseConnection dc, String mode);

	String getImplementationId();

	Object processOperation(operation op, OperationRequest request) throws UnsupportedOperationException;

	int DEFAULT_ID = 0;

	Connection getConnection(DatabaseConnection dc, String mode) throws TestExecutionError;
	Connection getConnection(DatabaseConnection dc, String mode, int identifier) throws TestExecutionError;

	void prepareConnectionForInsert(Connection connection, RelationalDataSet target, DatabaseConnection dc, String mode) throws Exception;

	void returnConnection(Connection conn, DatabaseConnection dc, String mode, int id) throws TestExecutionError;
	String getDefaultSchema(DatabaseConnection dc, String mode);

	void dispose();
}